;; $Id: dbbibl.dsl 0.90 1997/08/18 12:38:27 nwalsh Exp $

;; ......................... BIBLIOGRAPHIC DATA .........................

(define (author-string #!optional (author (current-node)))
  ;; Return a formatted string representation of the contents of:
  ;; AUTHOR:
  ;;   Handles *only* Honorific, FirstName, SurName, and Lineage.
  ;;   Handles *only* the first of each.
  ;;   Format is "Honorific. FirstName SurName, Lineage"
  ;; CORPAUTHOR:
  ;;   returns (data corpauthor)
  (let* ((h_nl (select-elements (descendants author) "HONORIFIC"))
	 (f_nl (select-elements (descendants author) "FIRSTNAME"))
	 (s_nl (select-elements (descendants author) "SURNAME"))
	 (l_nl (select-elements (descendants author) "LINEAGE"))
	 (has_h (not (node-list-empty? h_nl)))
	 (has_f (not (node-list-empty? f_nl)))
	 (has_s (not (node-list-empty? s_nl)))
	 (has_l (not (node-list-empty? l_nl))))
    (if (equal? (gi author) "AUTHOR")
	(string-append
	 (if has_h (string-append (data (node-list-first h_nl)) ".") "")
	 (if has_f (string-append 
		    (if has_h " " "") 
		    (data (node-list-first f_nl))) "")
	 (if has_s (string-append 
		    (if (or has_h has_f) " " "") 
		    (data (node-list-first s_nl))) "")
	 (if has_l (string-append ", " (data (node-list-first l_nl))) ""))
	(data author))))

(define (author-list-string #!optional (author (current-node)))
  ;; Return a formatted string representation of the contents of AUTHOR
  ;; *including appropriate punctuation* if the AUTHOR occurs in a list
  ;; of AUTHORs in an AUTHORGROUP:
  ;;
  ;;   John Doe
  ;; or
  ;;   John Doe and Jane Doe
  ;; or
  ;;   John Doe, Jane Doe, and A. Nonymous
  ;;

  (let* ((author-node-list (select-elements
			    (descendants 
			     (ancestor "AUTHORGROUP")) "AUTHOR"))
	 (corpauthor-node-list (select-elements
				(descendants 
				 (ancestor "AUTHORGROUP")) "CORPAUTHOR"))
	 (author-count (if (have-ancestor? "AUTHORGROUP" author)
			   (+ (node-list-length author-node-list)
			      (node-list-length corpauthor-node-list))
			   1)))
    (string-append
     (if (and (> author-count 1)
	      (last-sibling? author))
	 (string-append %gentext-and% " ")
	 "")
     (author-string)
     (if (and (> author-count 2)
	      (not (last-sibling? author)))
	 ", "
	 (if (> author-count 1)
	     " "
	     "")))))

(element BIBLIOGRAPHY ($component$))
(element (BIBLIOGRAPHY TITLE) ($comptitle$))

(element BIBLIODIV ($section$))
(element (BIBLIODIV TITLE) ($sectitle$))

(element BIBLIOENTRY (empty-sosofo))
(element BIBLIOMISC (empty-sosofo))
(element BOOKBIBLIO (empty-sosofo))

(element ACKNO ($charseq$))
(element STREET ($charseq$))
(element POB ($charseq$))
(element POSTCODE ($charseq$))
(element CITY ($charseq$))
(element STATE ($charseq$))
(element COUNTRY ($charseq$))
(element PHONE ($charseq$))
(element FAX ($charseq$))
(element OTHERADDR ($charseq$))
(element AFFILIATION ($charseq$))
(element SHORTAFFIL ($charseq$))
(element JOBTITLE ($charseq$))
(element ORGDIV ($charseq$))
(element ARTPAGENUMS ($charseq$))

(element AUTHOR
  (make sequence
    (literal (author-list-string))))

(element AUTHORGROUP (process-children))

(element COLLAB (process-children))
(element COLLABNAME ($charseq$))
(element AUTHORINITIALS ($charseq$))
(element CONFGROUP (process-children))
(element CONFDATES ($charseq$))
(element CONFTITLE ($charseq$))
(element CONFNUM ($charseq$))
(element CONFSPONSOR ($charseq$))
(element CONTRACTNUM ($charseq$))
(element CONTRACTSPONSOR ($charseq$))

(element COPYRIGHT
  (make paragraph
    use: para-style
    (make sequence
      (literal (gentext-element-name (gi (current-node))))
      (literal " ")
      (literal (dingbat "copyright"))
      (literal " ")
      (process-children-trim))))

(element YEAR
  (make sequence
    (process-children)
    (if (not (last-sibling? (current-node)))
	(literal ", ")
	(literal (string-append " " %gentext-by% " ")))))

(element HOLDER ($charseq$))

(element CORPAUTHOR
  (make sequence
    (literal (author-list-string))))

(element CORPNAME ($charseq$))
(element DATE ($charseq$))
(element EDITION ($charseq$))
(element EDITOR ($charseq$))
(element ISBN ($charseq$))
(element ISSN ($charseq$))
(element INVPARTNUMBER ($charseq$))
(element ISSUENUM ($charseq$))

(element LEGALNOTICE (process-children))

(element MODESPEC (empty-sosofo))

(element ORGNAME ($charseq$))

(element OTHERCREDIT ($charseq$))
(element PAGENUMS ($charseq$))
(element CONTRIB ($charseq$))

(element FIRSTNAME ($charseq$))
(element HONORIFIC ($charseq$))
(element LINEAGE ($charseq$))
(element OTHERNAME ($charseq$))
(element SURNAME ($charseq$))

(element PRINTHISTORY (empty-sosofo))
(element PRODUCTNAME ($charseq$))
(element PRODUCTNUMBER ($charseq$))
(element PUBDATE ($charseq$))
(element PUBLISHER (process-children))
(element PUBLISHERNAME ($charseq$))
(element PUBSNUMBER ($charseq$))
(element RELEASEINFO (empty-sosofo))
(element REVISION ($charseq$))
(element REVNUMBER ($charseq$))
(element REVREMARK ($charseq$))
(element SERIESVOLNUMS ($charseq$))
(element VOLUMENUM ($charseq$))

(element REVHISTORY
  (make sequence
    (make paragraph
      use: title-style
      font-family-name: %title-font-family%
      font-weight: 'bold
      space-before: (* (HSIZE 3) %head-before-factor%)
      space-after: (* (HSIZE 1) %head-before-factor%)
      (literal (gentext-element-name (gi (current-node)))))
    (make table
      before-row-border: #f
      (process-children))))

(element (REVHISTORY REVISION)
  (let ((revnumber (select-elements (descendants (current-node)) "REVNUMBER"))
	(revdate   (select-elements (descendants (current-node)) "DATE"))
	(revauthor (select-elements (descendants (current-node)) "AUTHORINITIALS"))
	(revremark (select-elements (descendants (current-node)) "REVREMARK")))
    (make sequence
      (make table-row
	(make table-cell
	  column-number: 1
	  n-columns-spanned: 1
	  n-rows-spanned: 1
	  (if (not (node-list-empty? revnumber))
	      (make paragraph
		(make sequence
		  (literal (gentext-element-name-space (gi (current-node))))
		  (process-node-list revnumber)))
	      (empty-sosofo)))
	(make table-cell
	  column-number: 2
	  n-columns-spanned: 1
	  n-rows-spanned: 1
	  (if (not (node-list-empty? revdate))
	      (make paragraph
		(process-node-list revdate))
	      (empty-sosofo)))
	(make table-cell
	  column-number: 3
	  n-columns-spanned: 1
	  n-rows-spanned: 1
	  (if (not (node-list-empty? revauthor))
	      (make paragraph
		(make sequence
		  (literal "Revised by: ")
		  (process-node-list revauthor)))
	      (empty-sosofo))))
      (make table-row
	cell-after-row-border: #f
	(make table-cell
	  column-number: 1
	  n-columns-spanned: 3
	  n-rows-spanned: 1
	  (if (not (node-list-empty? revremark))
	      (make paragraph
		space-after: %block-sep%
		(process-node-list revremark))
	      (empty-sosofo)))))))

(element (REVISION REVNUMBER) (process-children-trim))
(element (REVISION DATE) (process-children-trim))
(element (REVISION AUTHORINITIALS) (process-children-trim))
(element (REVISION REVREMARK) (process-children-trim))
