<!DOCTYPE style-sheet PUBLIC "-//James Clark//DTD DSSSL Style Sheet//EN">

<style-specification id="docbook-l10n-usen">

;; $Id: dbl1usen.dsl 0.91 1997/08/28 14:09:41 nwalsh Exp $

;; ----------------------------- Localization -----------------------------

;; If you create a new version of this file, please send it to
;; Norman Walsh, norm@berkshire.net


;; The generated text for cross references to elements.  See dblink.dsl
;; for a discussion of how substitution is performed on the %x and #x
;; keywords.
;;
(define (gentext-xref-strings giname)
  (case giname
    (("APPENDIX") "Appendix %n")
    (("CHAPTER") "Chapter %n")
    (("EXAMPLE") "Example %n")
    (("FIGURE") "Figure %n")
    (("PART") "Part %n")
    (("PREFACE") "%t")
    (("PROCEDURE") "Procedure %n, %t")
    (("SECT1") (if %section-autolabel% 
		   "Section %n" 
		   "the section called %t#c"))
    (("SECT2") (if %section-autolabel% 
		   "Section %n" 
		   "the section called %t#c"))
    (("SECT3") (if %section-autolabel% 
		   "Section %n" 
		   "the section called %t#c"))
    (("SECT4") (if %section-autolabel% 
		   "Section %n" 
		   "the section called %t#c"))
    (("SECT5") (if %section-autolabel% 
		   "Section %n" 
		   "the section called %t#c"))
    (("STEP") "step %n#k")
    (("TABLE") "Table %n")
    (else (string-append "[xref to " 
			 (if giname 
			     giname
			     "non-existant element")
			 " unsupported]"))))


;; Indirection in the generated text for cross references.  The first
;; element of the list is used for cross references across element
;; boundries, the second element is used for cross references within
;; the same element
;;
(define (gentext-xref-strings-indirect key)
  (case key
    (("b") '("" ""))
    (("c") '(" in %cg %cn" ""))
    (("d") '("" ""))
    (("k") '(" in %kg %kn" ""))
    (("s") '("" ""))))


;; Should the TOC come first or last?
;;
(define %generate-toc-in-front% #t)

;; gentext-element-name returns the generated text that should be 
;; used to make reference to the selected element.
;;
(define (gentext-element-name giname)
  (case giname
    (("ABSTRACT") "Abstract")
    (("APPENDIX") "Appendix")
    (("BIBLIOGRAPHY") "Bibliography")
    (("CAUTION") "CAUTION")
    (("CHAPTER") "Chapter")
    (("COPYRIGHT") "Copyright")
    (("EXAMPLE") "Example")
    (("FIGURE") "Figure")
    (("GLOSSARY") "Glossary")
    (("IMPORTANT") "IMPORTANT")
    (("INDEX") "Index")
    (("NOTE") "NOTE")
    (("PART") "Part")
    (("PREFACE") "Preface")
    (("PROCEDURE") "Procedure")
    (("REFNAME") "Name")
    (("REVHISTORY") "Revision History")
    (("REVISION") "Revision")
    (("SECT1") "Section")
    (("SECT2") "Section")
    (("SECT3") "Section")
    (("SECT4") "Section")
    (("SECT5") "Section")
    (("STEP") "step")
    (("TABLE") "Table")
    (("TIP") "TIP")
    (("TOC") "Table of Contents")
    (("WARNING") "WARNING")
    (else (string-append "UNEXPECTED-ELEMENT-NAME: " giname))))

;; gentext-element-name-space returns gentext-element-name with a 
;; trailing space, if gentext-element-name isn't "".
;;
(define (gentext-element-name-space giname)
  (string-with-space (gentext-element-name giname)))

;; gentext-intra-label-sep returns the seperator to be inserted
;; between multiple occurances of a label (or parts of a label)
;; for the specified element.  Most of these are for enumerated
;; labels like "Figure 2-4", but this function is used elsewhere
;; (e.g. REFNAME) with a little abuse.
;;
(define (gentext-intra-label-sep giname)
  (case giname
    (("EXAMPLE") "-")
    (("FIGURE") "-")
    (("PROCEDURE") ".")
    (("REFNAME") ", ")
    (("REFSECT1") ".")
    (("REFSECT2") ".")
    (("REFSECT3") ".")
    (("SECT1") ".")
    (("SECT2") ".")
    (("SECT3") ".")
    (("SECT4") ".")
    (("SECT5") ".")
    (("STEP") ".")
    (("TABLE") "-")
    (("_pagenumber") "-")  ;; page number psuedo element
    (else "")))

;; gentext-label-title-sep returns the seperator to be inserted
;; between a label and the text following the label for the
;; specified element.  Most of these are for use between
;; enumerated labels and titles like "1. Chapter One Title", but
;; this function is used elsewhere (e.g. NOTE) with a little
;; abuse.
;;
(define (gentext-label-title-sep giname)
  (case giname
    (("ABSTRACT") ": ")
    (("APPENDIX") ". ")
    (("CAUTION") "")
    (("CHAPTER") ". ")
    (("EXAMPLE") ". ")
    (("FIGURE") ". ")
    (("IMPORTANT") ": ")
    (("NOTE") ": ")
    (("ORDEREDLIST") ". ")
    (("PART") ". ")
    (("PROCEDURE") ". ")
    (("PREFIX") ". ")
    (("REFSECT1") ". ")
    (("REFSECT2") ". ")
    (("REFSECT3") ". ")
    (("SECT1") ". ")
    (("SECT2") ". ")
    (("SECT3") ". ")
    (("SECT4") ". ")
    (("SECT5") ". ")
    (("STEP") ":")
    (("TABLE") ". ")
    (("TIP") ": ")
    (("WARNING") "")
    (else "")))

(define ($lot-title$ lotgi)
  (cond ((string=? "TABLE"   lotgi) "List of Tables")
	((string=? "EXAMPLE" lotgi) "List of Examples")
	((string=? "FIGURE"  lotgi) "List of Figures")
	(else "List of ???")))

(define %gentext-by% "by") ;; e.g. Copyright 1997 "by" A. Nonymous
                           ;; Authored "by" Jane Doe

(define %gentext-page% "Page")

(define %gentext-start-quote%  (dingbat "ldquo"))

(define %gentext-end-quote%  (dingbat "rdquo"))

(define %gentext-and% "and")

</style-specification>
