;; $Id: dblink.dsl 0.91 1997/08/28 14:09:41 nwalsh Exp $

;; ========================= LINKS AND ANCHORS ==========================

(element LINK ($charseq$))
(element OLINK ($charseq$))
(element ULINK ($charseq$))
(element FOOTNOTEREF (empty-sosofo))
(element ANCHOR (empty-sosofo))
(element BEGINPAGE (empty-sosofo))

;; From the DocBook V3.0 Reference entry for element XREF:
;;
;; Description
;;
;;   Cross reference link to another part of the document. XRef is empty,
;;   and has common, Linkend, and Endterm attributes.
;;
;;   Processing Expectations
;;
;;   XRef must have a Linkend, but the Endterm is optional. If it is used,
;;   the content of the element it points to is displayed as the text of
;;   the cross reference; if it is absent, the XRefLabel of the
;;   cross-referenced object is displayed.
;;
;; If neither the ENDTERM nor the XREFLABEL is present, then the cross
;; reference text is taken from the (gentext-xref-strings) function
;; in the localization file, like this
;; 
;; A cross reference to an element, the target, begins with the
;; text returned by (gentext-xref-strings (gi target)).  Within
;; that text, the following substitutions are made:
;; 
;; %p is replaced by the number of the page on which target occurs
;; %[x]g is replaced by the (gentext-element-name)
;; %[x]n is replaced by the label
;; %[x]t is replaced by the title
;; 
;; Where "x" is either absent, in which case the target is used, or
;; one of the following:
;; 
;; b = the ancestral book
;; c = the ancestral component
;; d = the ancestral division
;; k = the ancestral block
;; s = the ancestral section
;; 
;; So, %cn is replaced by the label (number) of the chapter that
;; contains target.  %st is replaced by the title of the section
;; that contains target, %g is replaced by the
;; (gentext-element-name) of the target, etc.
;; 
;; What elements constitute a book, component, division, block, and
;; section are defined by the lists *-element-list.
;; 
;; As if this wasn't confusing enough, _one_ additional level of
;; indirection is available.  If one of the special symbols, #b,
;; #c, #d, #k, or #s occurs in the (gentext-xref-strings) text,
;; then it will be replaced by the appropriate
;; (gentext-xref-strings-indirect) text depending on whether or not
;; the target and the reference occur in the same element.
;; 
;; Here's a concrete example:
;; 
;; Given
;; 
;; (define (gentext-xref-strings-indirect key)
;;   (case key
;;     (("k") '(" in %kg %kn" ""))
;;     (("s") '(" in %cg %cn" " in this %cg"))))
;; 
;; and
;; 
;; (define (gentext-xref-strings giname)
;;   (case giname
;;     (("STEP") "step %n#k")
;;     (("SECT1") "the section called %t#s")))
;; 
;; A cross reference to a step in the same procedure as the reference
;; will use the string "step %n" as the gentext-xref-strings text.
;; A cross reference to a step in another procedure will use the
;; string "step %n in %kg %kn".
;; 
;; So, a reference from step 5 to step 3 in the same procedure will
;; appear as "step 3".  A reference to step 6 in the third
;; procedure in the fourth chapter from some other place will
;; appear as "step 6 in procedure 4.3".
;; 
;; Likewise a reference to another section in the current chapter
;; will appear like this "the section called target-title in this
;; Chapter", and a reference to a section in an appendix will
;; appear like this "the section called target-title in Appendix
;; B".
;; 
;; ======================================================================

(define book-element-list 
  (list "BOOK"))

(define division-element-list
  (list "PART"))

(define component-element-list
  (list "PREFACE" "CHAPTER" "APPENDIX" 
	"ARTICLE"
	"GLOSSARY" "BIBLIOGRAPHY" "INDEX"
	"REFERENCE" "REFENTRY"
	"BOOK")) ;; just in case nothing else matches...

(define section-element-list
  (list "SECT1" "SECT2" "SECT3" "SECT4" "SECT5"
	"REFSECT1" "REFSECT2" "REFSECT3"))

(define block-element-list
  (list "EXAMPLE" "FIGURE" "TABLE" "PROCEDURE"))

;; ======================================================================

(element XREF
  (let* ((endterm (attribute-string "ENDTERM"))
	 (linkend (attribute-string "LINKEND"))
	 (target  (element-with-id linkend)))
    (if (node-list-empty? target)
	(literal "[xref to non-existent element '" linkend "']")
	(if endterm
	    (if (node-list-empty? (element-with-id endterm))
		(literal "[xref endterm to non-existent element '" endterm "']")
		(with-mode xref-endterm-mode
		  (process-node-list (element-with-id endterm))))
	    (if (attribute-string "XREFLABEL" target)
		(xreflabel-sosofo (attribute-string "XREFLABEL" target))
		(auto-xref target))))))

(mode xref-endterm-mode
  (default
    (make sequence
      font-posture: 'italic
      (process-children-trim))))

(define (xreflabel-sosofo xreflabel)
  (make sequence
    font-posture: 'italic
    (literal xreflabel)))

;; ======================================================================

(define (auto-xref target)
  (let* ((cont-blok (ancestral-member target block-element-list))
	 (cont-sect (ancestral-member target section-element-list))
	 (cont-comp (ancestral-member target component-element-list))
	 (cont-divn (ancestral-member target division-element-list))
 	 (cont-book (ancestral-member target book-element-list))
	 (substitute (list
		      (list "%bg" (element-gi-sosofo cont-book))
		      (list "%bn" (element-label-sosofo cont-book))
		      (list "%bt" (element-title-xref-sosofo cont-book))
		      (list "%cg" (element-gi-sosofo cont-comp))
		      (list "%cn" (element-label-sosofo cont-comp))
		      (list "%ct" (element-title-xref-sosofo cont-comp))
		      (list "%dg" (element-gi-sosofo cont-divn))
		      (list "%dn" (element-label-sosofo cont-divn))
		      (list "%dt" (element-title-xref-sosofo cont-divn))
		      (list "%g"  (element-gi-sosofo target))
		      (list "%kg" (element-gi-sosofo cont-blok))
		      (list "%kn" (element-label-sosofo cont-blok))
		      (list "%kt" (element-title-xref-sosofo cont-blok))
		      (list "%n"  (element-label-sosofo target))
		      (list "%p"  (element-page-number-sosofo target))
		      (list "%sg" (element-gi-sosofo cont-sect))
		      (list "%sn" (element-label-sosofo cont-sect))
		      (list "%st" (element-title-xref-sosofo cont-sect))
		      (list "%t"  (element-title-xref-sosofo target))))
	 (text        (subst-xref-strings-indirect 
		       (gentext-xref-strings (gi target))
		       (current-node)
		       target))
	 (tlist   (match-split-list text (assoc-objs substitute))))
    (make sequence
      (string-list-sosofo tlist substitute))))

;; ======================================================================

(define (subst-xref-strings-indirect string reference referent)
  (let* ((rnce-blok (ancestral-member reference block-element-list))
	 (rnce-sect (ancestral-member reference section-element-list))
	 (rnce-comp (ancestral-member reference component-element-list))
	 (rnce-divn (ancestral-member reference division-element-list))
 	 (rnce-book (ancestral-member reference book-element-list))

	 (rent-blok (ancestral-member referent block-element-list))
	 (rent-sect (ancestral-member referent section-element-list))
	 (rent-comp (ancestral-member referent component-element-list))
	 (rent-divn (ancestral-member referent division-element-list))
 	 (rent-book (ancestral-member referent book-element-list))

	 (title     (element-title-xref-sosofo referent))

	 (b (if (node-list-empty? rent-book)
		""
		(if (node-list=? rnce-book rent-book)
		    (car (cdr (gentext-xref-strings-indirect "b")))
		    (car (gentext-xref-strings-indirect "b")))))

	 (c (if (node-list-empty? rent-comp)
		""
		(if (node-list=? rnce-comp rent-comp)
		    (car (cdr (gentext-xref-strings-indirect "c")))
		    (car (gentext-xref-strings-indirect "c")))))

	 (d (if (node-list-empty? rent-divn)
		""
		(if (node-list=? rnce-divn rent-divn)
		    (car (cdr (gentext-xref-strings-indirect "d")))
		    (car (gentext-xref-strings-indirect "d")))))

	 (k (if (node-list-empty? rent-blok)
		""
		(if (node-list=? rnce-blok rent-blok)
		    (car (cdr (gentext-xref-strings-indirect "k")))
		    (car (gentext-xref-strings-indirect "k")))))

	 (s (if (node-list-empty? rent-sect)
		""
		(if (node-list=? rnce-sect rent-sect)
		    (car (cdr (gentext-xref-strings-indirect "s")))
		    (car (gentext-xref-strings-indirect "s"))))))

    (string-replace-list string 
			 (list "#b" b "#c" c "#d" d "#k" k "#s" s))))

;; ======================================================================

(define (book-autolabel nd #!optional (force-label? #f))
  (format-number (child-number nd) "1"))

(define (part-autolabel nd #!optional (force-label? #f))
  (format-number (child-number nd) "I"))

(define (preface-autolabel nd #!optional (force-label? #f))
  "")

(define (article-autolabel nd #!optional (force-label? #f))
  "")

(define (chapter-autolabel nd #!optional (force-label? #f))
  (format-number (element-number nd) "1"))

(define (appendix-autolabel nd #!optional (force-label? #f))
  (format-number (child-number nd) "A"))

(define (bibliography-autolabel nd #!optional (force-label? #f))
  "")

(define (glossary-autolabel nd #!optional (force-label? #f))
  "")

(define (index-autolabel nd #!optional (force-label? #f))
  "")

(define (reference-autolabel nd #!optional (force-label? #f))
  (format-number (child-number nd) "1"))

(define (refentry-autolabel nd #!optional (force-label? #f))
  (format-number (child-number nd) "1"))

(define (section-autolabel nd #!optional (force-label? #f))
  (let* ((isep (gentext-intra-label-sep (gi nd)))
	 (haschn (not (node-list-empty? (ancestor "CHAPTER" nd))))
	 (hasapn (not (node-list-empty? (ancestor "APPENDIX" nd))))
	 (prefix (cond
		  (haschn (string-append 
			   (element-label (ancestor "CHAPTER" nd)) isep))
		  (hasapn (string-append 
			   (element-label (ancestor "APPENDIX" nd)) isep))
		  (else ""))))
    (if (or force-label? %section-autolabel%)
	(case (gi nd)
	  (("SECT1") (string-append prefix 
				    (format-number (child-number nd) "1")))
	  (("SECT2") (string-append (element-label 
				     (ancestor "SECT1" nd) force-label?)
				    isep 
				    (format-number (child-number nd) "1")))
	  (("SECT3") (string-append (element-label 
				     (ancestor "SECT2" nd) force-label?)
				    isep 
				    (format-number (child-number nd) "1")))
	  (("SECT4") (string-append (element-label 
				     (ancestor "SECT3" nd) force-label?)
				    isep 
				    (format-number (child-number nd) "1")))
	  (("SECT5") (string-append (element-label 
				     (ancestor "SECT4" nd) force-label?)
				    isep 
				    (format-number (child-number nd) "1")))
	  (else (string-append (gi nd) " IS NOT A SECTION!")))
	"")))
  
(define (refsection-autolabel nd #!optional (force-label? #f))
  (let* ((s1n (ancestor-child-number "REFSECT1"))
	 (s2n (ancestor-child-number "REFSECT2"))
	 (s3n (ancestor-child-number "REFSECT3"))
	 (isep (gentext-intra-label-sep (gi nd))))
    (string-append
     (if s1n (string-append isep (FNUM s1n)) "")
     (if s2n (string-append isep (FNUM s2n)) "")
     (if s3n (string-append isep (FNUM s3n)) ""))))

(define (step-autolabel nd #!optional (force-label? #f))
  ($proc-step-xref-number$ nd))

(define (block-autolabel nd #!optional (force-label? #f))
  (let* ((chn (element-label (ancestor "CHAPTER" nd)))
	 (apn (element-label (ancestor "APPENDIX" nd)))
	 (bkn (format-number (component-child-number nd) "1")))
    (cond 
     (chn (string-append chn (gentext-intra-label-sep (gi nd)) bkn))
     (apn (string-append chn (gentext-intra-label-sep (gi nd)) bkn))
     (else bkn))))

;; For all elements, if a LABEL attribute is present, that is the label
;; that they get.  Otherwise:
;; BOOK gets the Book volume, by book-autolabel
;; PREFACE gets "", by preface-autolabel
;; CHAPTER gets the Chapter number, by chapter-autolabel
;; APPENDIX gets the Appendix letter, by appendix-autolabel
;; REFERENCE gets "", by reference-autolabel
;; REFENTRY gets "", by refentry-autolabel
;; SECT* gets the nested section number (e.g., 1.3.5), by section-autolabel
;; REFSECT* gets the nested section number, by refsection-autolabel
;; everything else gets numbered by block-autolabel
;;
(define (element-label nd #!optional (force-label? #f))
  (if (node-list-empty? nd)
      ""
      (let ((label (attribute-string "LABEL" nd)))
	(if label
	    label
	    (case (gi nd)
	      ;; Use a seperately defined assoc list?
	      (("BOOK") (book-autolabel nd force-label?))
	      (("PART") (part-autolabel nd force-label?))
	      (("ARTICLE") (article-autolabel nd force-label?))
	      (("PREFACE") (preface-autolabel nd force-label?))
	      (("CHAPTER") (chapter-autolabel nd force-label?))
	      (("APPENDIX") (appendix-autolabel nd force-label?))
	      (("BIBLIOGRAPHY") (bibliography-autolabel nd force-label?))
	      (("GLOSSARY") (glossary-autolabel nd force-label?))
	      (("INDEX") (index-autolabel nd force-label?))
	      (("REFERENCE") (reference-autolabel nd force-label?))
	      (("REFENTRY") (refentry-autolabel nd force-label?))
	      (("SECT1") (section-autolabel nd force-label?))
	      (("SECT2") (section-autolabel nd force-label?))
	      (("SECT3") (section-autolabel nd force-label?))
	      (("SECT4") (section-autolabel nd force-label?))
	      (("SECT5") (section-autolabel nd force-label?))
	      (("REFSECT1") (refsection-autolabel nd force-label?))
	      (("REFSECT2") (refsection-autolabel nd force-label?))
	      (("REFSECT3") (refsection-autolabel nd force-label?))
	      (("STEP") (step-autolabel nd force-label?))
	      (else (block-autolabel nd force-label?)))))))

;; Returns the element label as a sosofo
;;
(define (element-label-sosofo nd #!optional (force-label? #f))
  (if (string=? (element-label nd force-label?) "")
      (empty-sosofo)
      (make sequence
	(literal (element-label nd force-label?)))))

;; ======================================================================

(define (book-title-sosofo nd)
  (let* ((bookinfo (select-elements (children nd) "BOOKINFO"))
	 (titles (node-list 
		  (if bookinfo
		      (select-elements (children bookinfo) "TITLE")
		      (empty-sosofo))
		  (select-elements (children nd) "TITLE"))))
    (if (node-list-empty? titles)
	(empty-sosofo)
	(with-mode xref-title-mode
	  (process-node-list (node-list-first titles))))))

(define (part-title-sosofo nd)
  (let ((titles (select-elements (children nd) "TITLE")))
    (if (node-list-empty? titles)
	(empty-sosofo)
	(with-mode xref-title-mode
	  (process-node-list (node-list-first titles))))))

(define (article-title-sosofo nd)
  (let* ((artheader (select-elements (children nd) "ARTHEADER"))
	 (titles (select-elements (children artheader) "TITLE")))
    (if (node-list-empty? titles)
	(empty-sosofo)
	(with-mode xref-title-mode
	  (process-node-list (node-list-first titles))))))

(define (preface-title-sosofo nd)
  (let ((titles (select-elements (children nd) "TITLE")))
    (if (node-list-empty? titles)
	(empty-sosofo)
	(with-mode xref-title-mode
	  (process-node-list (node-list-first titles))))))

(define (chapter-title-sosofo nd)
  (let ((titles (select-elements (children nd) "TITLE")))
    (if (node-list-empty? titles)
	(empty-sosofo)
	(with-mode xref-title-mode
	  (process-node-list (node-list-first titles))))))

(define (appendix-title-sosofo nd)
  (let ((titles (select-elements (children nd) "TITLE")))
    (if (node-list-empty? titles)
	(empty-sosofo)
	(with-mode xref-title-mode
	  (process-node-list (node-list-first titles))))))

(define (optional-title-sosofo nd)
  (let ((titles (select-elements (children nd) "TITLE")))
    (if (node-list-empty? titles)
	(literal (gentext-element-name (gi nd)))
	(with-mode xref-title-mode
	  (process-node-list (node-list-first titles))))))

(define (glossary-title-sosofo nd)
  (optional-title-sosofo nd))

(define (bibliography-title-sosofo nd)
  (optional-title-sosofo nd))

(define (index-title-sosofo nd)
  (optional-title-sosofo nd))

(define (reference-title-sosofo nd)
  (let ((titles (select-elements (children nd) "TITLE")))
    (if (node-list-empty? titles)
	(empty-sosofo)
	(with-mode xref-title-mode
	  (process-node-list (node-list-first titles))))))

;; Returns either the REFENTRYTITLE or the first REFNAME.
;;
(define (refentry-title-sosofo nd)
  (let* ((refmeta (select-elements (descendants nd) "REFMETA"))
	 (refttl  (select-elements (descendants refmeta) "REFENTRYTITLE"))
	 (refndiv (select-elements (descendants nd) "REFNAMEDIV"))
	 (refname (select-elements (descendants refndiv) "REFNAME")))
    (if (node-list-empty? refttl)
	(if (node-list-empty? refname)
	    (empty-sosofo)
	    (with-mode xref-title-mode
	      (process-node-list (node-list-first refname))))
	(with-mode xref-title-mode
	  (process-node-list (node-list-first refttl))))))

(define (section-title-sosofo nd)
  (let ((titles (select-elements (children nd) "TITLE")))
    (if (node-list-empty? titles)
	(empty-sosofo)
	(with-mode xref-title-mode
	  (process-node-list (node-list-first titles))))))

(define (refsection-title-sosofo nd)
  (let ((titles (select-elements (children nd) "TITLE")))
    (if (node-list-empty? titles)
	(empty-sosofo)
	(with-mode xref-title-mode
	  (process-node-list (node-list-first titles))))))

(define (block-title-sosofo nd)
  (let ((titles (select-elements (children nd) "TITLE")))
    (if (node-list-empty? titles)
	(empty-sosofo)
	(with-mode xref-title-mode
	  (process-node-list (node-list-first titles))))))

;; Returns the title of the element as a sosofo.
;;
(define (element-title-sosofo nd)
  (if (node-list-empty? nd)
      (empty-sosofo)
      (case (gi nd)
	;; Use a seperately defined assoc list?
	(("BOOK") (book-title-sosofo nd))
	(("PART") (part-title-sosofo nd))
	(("ARTICLE") (article-title-sosofo nd))
	(("PREFACE") (preface-title-sosofo nd))
	(("CHAPTER") (chapter-title-sosofo nd))
	(("APPENDIX") (appendix-title-sosofo nd))
	(("GLOSSARY") (glossary-title-sosofo nd))
	(("BIBLIOGRAPHY") (bibliography-title-sosofo nd))
	(("INDEX") (index-title-sosofo nd))
	(("REFERENCE") (reference-title-sosofo nd))
	(("REFENTRY") (refentry-title-sosofo nd))
	(("SECT1") (section-title-sosofo nd))
	(("SECT2") (section-title-sosofo nd))
	(("SECT3") (section-title-sosofo nd))
	(("SECT4") (section-title-sosofo nd))
	(("SECT5") (section-title-sosofo nd))
	(("REFSECT1") (refsection-title-sosofo nd))
	(("REFSECT2") (refsection-title-sosofo nd))
	(("REFSECT3") (refsection-title-sosofo nd))
	(else (block-title-sosofo nd)))))

;; Returns the title of the element as a sosofo, italicized for xref.
;;
(define (element-title-xref-sosofo nd)
  (make sequence
    font-posture: 'italic
    (element-title-sosofo nd)))

(mode xref-title-mode
  (default
    (process-children-trim)))

;; ======================================================================

;; Returns the element gi as a sosofo
;;
(define (element-gi-sosofo nd)
  (if (node-list-empty? nd)
      (empty-sosofo)
      (make sequence
	(literal (gentext-element-name (gi nd))))))

;; ======================================================================

(define (element-page-number-sosofo target)
  (with-mode pageno-mode
	(process-node-list target)))

(mode pageno-mode
  (default
    (current-node-page-number-sosofo)))

;; ======================================================================

